__version__ = "0.1.5"
__git_commiter_name__ = "Arnon Yaari"
__git_commiter_email__ = "arnony@infinidat.com"
__git_branch__ = '(HEAD detached at v0.1.5)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = 'a202cb5ea699685d7594f09323da38584b3964ca'
__git_head_subject__ = 'Finished release v0.1.5.'
__git_head_message__ = ''
__git_dirty_diff__ = ''
__git_commit_date__ = '2015-12-16 17:09:02'
